do
	local class = select(2,UnitClass("player"));
	if (class == "MAGE" or class == "WARLOCK" or class == "PRIEST") then
		return;
	end
end

local f = InfoPanel.AddPlugin();
local attackSpeed, critChance, attackPower, hasBattleShout, attackPowerMax;
local GetAP, GetCrit, GetSpeed;
local battleShout = GetSpellInfo(6673);

--------------------------------------------------------------------------------------------------------
--                                           Button Scripts                                           --
--------------------------------------------------------------------------------------------------------

-- OnEnter
local function OnEnter(self)
	self:MakeTip("Attack Power Info");

	GameTooltip:AddDoubleLine("Attack Speed",format("%.2f",attackSpeed),nil,nil,nil,1,1,1);
	GameTooltip:AddDoubleLine("Crit Chance",format("%.2f%%",critChance),nil,nil,nil,1,1,1);
	GameTooltip:AddDoubleLine("Hit Chance",format("%.2f%%",GetCombatRatingBonus(CR_HIT_MELEE)),nil,nil,nil,1,1,1);
	GameTooltip:AddDoubleLine("Armor Penetration",format("%.2f%%",GetArmorPenetration()),nil,nil,nil,1,1,1);
	GameTooltip:AddDoubleLine("Attack Power",attackPower,nil,nil,nil,1,1,1);
	GameTooltip:AddDoubleLine("Max Session AP",attackPowerMax,nil,nil,nil,attackPowerMax > attackPower and 0 or 1,1,attackPowerMax > attackPower and 0 or 1);

	local lowDmg, hiDmg, offlowDmg, offhiDmg, posBuff, negBuff, percentmod = UnitDamage("player");
	local _, offhandSpeed = GetSpeed("player")
	GameTooltip:AddLine(" ");
	GameTooltip:AddLine("Weapon Damage",1,1,1);
	GameTooltip:AddDoubleLine("Modifier",format("%.1f%%",percentmod * 100),nil,nil,nil,percentmod == 1 and 1 or 0,1,percentmod == 1 and 1 or 0);
	GameTooltip:AddDoubleLine("Mainhand",format("%.1f - %.1f",lowDmg,hiDmg),nil,nil,nil,1,1,1);
	if (OffhandHasWeapon()) then
		GameTooltip:AddDoubleLine("Offhand",format("%.1f - %.1f",offlowDmg,offhiDmg),nil,nil,nil,1,1,1);
	end
	GameTooltip:AddDoubleLine("Mainhand DPS",format("%.1f",(lowDmg + hiDmg) / 2 / attackSpeed),nil,nil,nil,1,1,1);
	if (OffhandHasWeapon()) then
		GameTooltip:AddDoubleLine("Offhand DPS",format("%.1f",(offlowDmg + offhiDmg) / 2 / offhandSpeed),nil,nil,nil,1,1,1);
	end

	GameTooltip:Show();
end

-- OnClick
local function OnClick(self,button)
	if (ChatFrameEditBox:IsShown()) then
		ChatFrameEditBox:Insert(format("%.2f%% Crit, %d AP",critChance,attackPower));
	end
end

--------------------------------------------------------------------------------------------------------
--                                               Events                                               --
--------------------------------------------------------------------------------------------------------

function f:UpdateDisplay()
	self:SetText("|cffffffff%.2f  |cff80c0ff%.2f%%  %s%d",attackSpeed,critChance,(hasBattleShout and "|cff52ff52" or "|cffff5252"),attackPower);
end

-- AP Change
function f:UNIT_ATTACK_POWER()
	local base, posBuff, negBuff = GetAP("player");
	attackPower = max(0,base + posBuff + negBuff);
	if (attackPower > attackPowerMax) then
		attackPowerMax = attackPower;
	end
	self:UpdateDisplay();
end
f.UNIT_RANGED_ATTACK_POWER = f.UNIT_ATTACK_POWER;

-- Attack Speed Change (show main hand only)
function f:UNIT_ATTACK_SPEED()
	attackSpeed = GetSpeed("player");
	self:UpdateDisplay();
end

-- Damage Mods Change (crit chance etc)
function f:PLAYER_DAMAGE_DONE_MODS()
	critChance = GetCrit();
	self:UpdateDisplay();
end

-- Check for BattleShout
function f:UNIT_AURA()
	hasBattleShout = (UnitAura("player",battleShout,nil,"HELPFUL") ~= nil);
	self:UpdateDisplay();
end

-- Player Died
function f:PLAYER_DEAD()
	hasBattleShout = nil;
	self:UpdateDisplay();
end

-- Login Only Event
function f:PLAYER_LOGIN(event)
	if (select(2,UnitClass("player")) ~= "WARRIOR") or (UnitAura("player",battleShout,nil,"HELPFUL")) then
		hasBattleShout = true;
	end

	local base, posBuff, negBuff = GetAP("player");
	attackPower = max(0,base + posBuff + negBuff);
	critChance = GetCrit();
	attackSpeed = GetSpeed("player");
	attackPowerMax = attackPower;

	self.PLAYER_LOGIN = nil;
	self:UpdateDisplay();
end

--------------------------------------------------------------------------------------------------------
--                                              Finalize                                              --
--------------------------------------------------------------------------------------------------------

do
	f.icon:SetTexture("Interface\\Icons\\Ability_Warrior_Battleshout");
	f:SetScript("OnEnter",OnEnter);
	f:SetScript("OnClick",OnClick);

	local _, class = UnitClass("player");
	GetAP = (class == "HUNTER" and UnitRangedAttackPower or UnitAttackPower);
	GetCrit = (class == "HUNTER" and GetRangedCritChance or GetCritChance);
	GetSpeed = (class == "HUNTER" and UnitRangedDamage or UnitAttackSpeed);

	if (class == "WARRIOR") then
		f:RegisterEvent("UNIT_AURA");
		f:RegisterEvent("PLAYER_DEAD");
	else
		f.UNIT_AURA = nil;
		f.PLAYER_DEAD = nil;
	end
	if (class == "HUNTER") then
		f:RegisterEvent("UNIT_RANGED_ATTACK_POWER");
	else
		f:RegisterEvent("UNIT_ATTACK_POWER");
	end
	f:RegisterEvent("PLAYER_DAMAGE_DONE_MODS");
	f:RegisterEvent("UNIT_ATTACK_SPEED");
	f:RegisterEvent("PLAYER_LOGIN");
end