local f = InfoPanel.AddPlugin();
local UPDATE_INTERVAL = 1;
local nextUpdate = 0;
local addons = {};
local fps, down, up, latency, mem;

local GetFramerate = GetFramerate;
local GetNetStats = GetNetStats;
local gcinfo = gcinfo;

--------------------------------------------------------------------------------------------------------
--                                             Functions                                              --
--------------------------------------------------------------------------------------------------------

local function SortAddonsFunc(a,b)
	return a.mem > b.mem;
end

-- OnUpdate
local function OnUpdate(self,elapsed)
	nextUpdate = (nextUpdate - elapsed);
	if (nextUpdate <= 0) then
		fps = GetFramerate();
		down, up, latency = GetNetStats();
		mem = gcinfo();

		self:SetText("|cff00ff00%.1f  |cffffd100%dms|r  %.2f Mb",fps,latency,mem / 1024);
		nextUpdate = UPDATE_INTERVAL;
	end
end

-- Mouse OnEnter
local function OnEnter(self)
	-- Normal Info
	if (not IsShiftKeyDown()) then
		self:MakeTip("Performance Info");

		GameTooltip:AddDoubleLine("Frame Rate",format("%.2f",fps),nil,nil,nil,1,1,1);

		GameTooltip:AddLine(" ");
		GameTooltip:AddDoubleLine("Bandwidth Down",format("%.2f bytes/s",down * 1024),nil,nil,nil,1,1,1);
		GameTooltip:AddDoubleLine("Bandwidth Up",format("%.2f bytes/s",up * 1024),nil,nil,nil,1,1,1);
		GameTooltip:AddDoubleLine("Latency",format("%d ms",latency),nil,nil,nil,1,1,1);

		local counter = 0;
		for _ in next, _G do
			counter = (counter + 1);
		end

		GameTooltip:AddLine(" ");
		GameTooltip:AddDoubleLine("Global Namespace Entries",counter,nil,nil,nil,1,1,1);
		GameTooltip:AddDoubleLine("UI Memory Usage",format("%d kb",mem),nil,nil,nil,1,1,1);
		GameTooltip:AddLine("<Press Shift for Detailed Addon Info>",0.5,0.5,0.5);
	-- Addon Details
	else
		UpdateAddOnMemoryUsage();
		local memTotal = 0;
		for i = 1, GetNumAddOns() do
			if (not addons[i]) then
				addons[i] = {};
			end
			addons[i].mem = GetAddOnMemoryUsage(i);
			addons[i].name = GetAddOnInfo(i);
			memTotal = (memTotal + addons[i].mem);
		end
		sort(addons,SortAddonsFunc);

		self:MakeTip("Addon Memory Usage ("..#addons..")");
		for index, addon in ipairs(addons) do
			if (addon.mem > 0) then
				GameTooltip:AddDoubleLine("|cffffffff"..format("%.2d|r ",index)..addon.name,format("%.2f kb",addon.mem),nil,nil,nil,1,1,1);
				if (GameTooltip:NumLines() > 50) then
					break;
				end
			end
		end
		GameTooltip:AddLine(strrep("-",58),1,1,1);
		GameTooltip:AddDoubleLine("Addon Memory Usage",format("%d kb",memTotal),nil,nil,nil,1,1,1);
		GameTooltip:AddDoubleLine("UI Memory Usage",format("%d kb",mem),nil,nil,nil,1,1,1);
	end

	GameTooltip:Show();
end

-- OnClick - Perform a GC
local function OnClick()
	UpdateAddOnMemoryUsage();
	local memBefore = gcinfo();
	collectgarbage();
	UpdateAddOnMemoryUsage();
	local memAfter = gcinfo();
	AzMsg(("|2Garbage Collection|r Before: |1%d|r kb, After: |1%d|r kb, Reduction: |1%d|r kb."):format(memBefore,memAfter,memBefore-memAfter));
end

--------------------------------------------------------------------------------------------------------
--                                              Finalize                                              --
--------------------------------------------------------------------------------------------------------

f.icon:SetTexture("Interface\\Icons\\Ability_Druid_LunarGuidance");
f:SetScript("OnUpdate",OnUpdate);
f:SetScript("OnEnter",OnEnter);
f:SetScript("OnClick",OnClick);