local _G = getfenv(0);

-- Config
local cfg;
local DefaultConfig = {
	topAnchor = false,
	defaultTip = false,
};

local lastPlugin;
local extraOffset = 34;

-- Init Specific Mod Vars
InfoPanel = {};
local f = CreateFrame("Frame",nil,UIParent);
local modName = "InfoPanel";

--------------------------------------------------------------------------------------------------------
--                                           Misc Functions                                           --
--------------------------------------------------------------------------------------------------------

-- AdjustAnchors
local function AdjustAnchors()
	-- InfoPanel
	f:ClearAllPoints();
	if (cfg.topAnchor) then
		f:SetPoint("TOPLEFT",-1,1);
		f:SetPoint("TOPRIGHT",1,1);
	else
		f:SetPoint("BOTTOMLEFT",-1,-1);
		f:SetPoint("BOTTOMRIGHT",1,-1);
	end
	-- MainMenuBar
	if (MainMenuBar:IsShown()) then
		MainMenuBar:ClearAllPoints();
		if (cfg.topAnchor) then
			MainMenuBar:SetPoint("BOTTOM");
		else
			MainMenuBar:SetPoint("BOTTOM",f,"TOP");
		end
	end
end

-- OnEvent
local function OnEvent(self,event)
	if (not InfoPanel_Config) then
		InfoPanel_Config = {};
	end
	cfg = InfoPanel_Config;
	setmetatable(cfg,{ __index = DefaultConfig });
	AdjustAnchors();
	OnEvent = nil;
end

-- Setup the Main Frame
f:SetHeight(26);
f:SetToplevel(1);
f:SetBackdrop({ bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = true, tileSize = 8, edgeSize = 8, insets = { left = 2, right = 2, top = 2, bottom = 2 } });
f:SetBackdropColor(0.1,0.22,0.35,1);
f:SetBackdropBorderColor(0.1,0.1,0.1,1);
f:Show();
f:SetScript("OnEvent",OnEvent);
f:RegisterEvent("VARIABLES_LOADED");

--------------------------------------------------------------------------------------------------------
--                                           Slash Handling                                           --
--------------------------------------------------------------------------------------------------------
_G["SLASH_"..modName.."1"] = "/ip";
SlashCmdList[modName] = function(cmd)
	-- Extract Paramters
	local param1, param2 = cmd:match("^([^%s]+)%s*(.*)$");
	param1 = (param1 and param1:lower() or cmd:lower());
	-- Show / Hide
	if (param1 == "show") then
		if (f:IsVisible()) then
			f:Hide();
		else
			f:Show();
		end
	-- Swap Anchor
	elseif (param1 == "anchor") then
		cfg.topAnchor = not cfg.topAnchor;
		AdjustAnchors();
	-- Tip
	elseif (param1 == "tip") then
		cfg.defaultTip = not cfg.defaultTip;
	-- Invalid or No Command
	else
		UpdateAddOnMemoryUsage();
		AzMsg(format("----- |2%s|r |1%s|r ----- |1%.2f |2kb|r -----",modName,GetAddOnMetadata(modName,"Version"),GetAddOnMemoryUsage(modName)));
		AzMsg("The following |2parameters|r are valid for this addon:");
		AzMsg(" |2anchor|r = Swap anchor position between top and bottom");
		AzMsg(" |2tip|r = Toggles between plugin anchored tip or default anchored tip");
	end
end

--------------------------------------------------------------------------------------------------------
--                                Global Chat Message Function (Rev 3)                                --
--------------------------------------------------------------------------------------------------------
if (not AZMSG_REV or AZMSG_REV < 3) then
	AZMSG_REV = 3;
	function AzMsg(text)
		DEFAULT_CHAT_FRAME:AddMessage(tostring(text):gsub("|1","|cffffff80"):gsub("|2","|cffffffff"),128/255,192/255,255/255);
	end
end

--------------------------------------------------------------------------------------------------------
--                                       Plugin Helper Functions                                      --
--------------------------------------------------------------------------------------------------------

-- Standard OnLeave Script
local function HideGTT()
	GameTooltip:Hide();
end

-- Standard OnEvent Script
local function Default_OnEvent(self,event,...)
	if (self[event]) then
		self[event](self,event,...);
	end
end

-- Quick SetText for Plugins to use
local function SetText(self,text,...)
	local oldWidth = self.text:GetWidth();
	-- text
	if (select("#",...) > 0) then
		self.text:SetFormattedText(text,...);
	else
		self.text:SetText(text);
	end
	-- width
	if (abs(oldWidth - self.text:GetWidth()) > 3) then
		self:SetWidth(self.text:GetWidth() + extraOffset);
	end
	-- tip
	if (GameTooltip:IsOwned(self)) and (self:GetScript("OnEnter")) then
		self:GetScript("OnEnter")(self);
	end
end

-- SetTextColor
local function SetTextColor(self,value,lowBoundry,highBoundry)
	if (value > highBoundry) then
		self.text:SetTextColor(0,1,0);
	elseif (value > lowBoundry) then
		self.text:SetTextColor(1,1,0);
	else
		self.text:SetTextColor(1,0,0);
	end
end

-- InitTip (Initializes the GTT for use)
local function MakeTip(self,header,extra,...)
	-- Set Tip Anchor
	local placement = (cfg.topAnchor and "ANCHOR_BOTTOM" or "ANCHOR_TOP"); -- Az: this is still not acceptable, but will have to do for now
	if (cfg.defaultTip) then
		GameTooltip_SetDefaultAnchor(GameTooltip,self);
	elseif (self.isRightAligned) then
		GameTooltip:SetOwner(self,placement.."RIGHT",4,0);
	else
		GameTooltip:SetOwner(self,placement.."LEFT");
	end
	-- Add Header
	if (not extra) then
		GameTooltip:AddLine(header,1,1,1,...);
	else
		GameTooltip:AddDoubleLine(header,extra,1,1,1,...);
	end
end

-- Add Plugin
function InfoPanel.AddPlugin(align)
	local frame = CreateFrame("Button",nil,f);
	frame:SetWidth(20);
	frame:SetHeight(f:GetHeight());
	frame.icon = frame:CreateTexture(nil,"OVERLAY");
	frame.icon:SetPoint("LEFT",4,0);
	frame.icon:SetWidth(18);
	frame.icon:SetHeight(18);
	frame.icon:SetTexCoord(0.07,0.93,0.07,0.93);
	frame.text = frame:CreateFontString(nil,"OVERLAY","GameFontHighlight");
	frame.text:SetPoint("LEFT",frame.icon,"RIGHT",4,0);
	frame:SetScript("OnLeave",HideGTT);
	frame:SetScript("OnEvent",Default_OnEvent);
	frame:EnableMouse(1);
	frame:RegisterForClicks("AnyUp");
	frame:Show();
	-- Functions
	frame.SetText = SetText;
	frame.SetTextColor = SetTextColor;
	frame.MakeTip = MakeTip;
	-- Anchor
	if (align == "right") then
		frame.isRightAligned = 1;
		frame:SetPoint("RIGHT",-8,0);
	elseif (lastPlugin) then
		frame:SetPoint("LEFT",lastPlugin,"RIGHT");
	else
		frame:SetPoint("LEFT",2,0);
	end
	lastPlugin = frame;
	-- Return
	return frame;
end